=begin

Copyright 2011, Andreas Eisenbarth
All Rights Reserved

Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies.

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESSED OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

Name:         FindProjectiveTextures.rb
Author:       Andreas Eisenbarth
Description:  This Plugin selects faces that SketchUp would export as duplicate/
              unique textures (because they have a so-called "projective" texture
              mapping).
Usage:        menu Plugins > 'Select textures that export as unique'
              If nothing is selected, it searches through the active context, 
              if something is selected, it only filters the selection.
              It is not recursive, so it won't dig into nested groups/components.
Version:      1.0
Date:         15.12.2012

=end
require 'sketchup.rb'



module AE


module AE::FindProjectiveTextures


unless file_loaded?(File.basename(__FILE__))
  cmd = UI::Command.new("Select textures that export as unique"){AE::FindProjectiveTextures.select_projective_textures}
	menu_plugins = UI.menu("Plugins")
	menu_plugins.add_item(cmd)
end


def self.select_projective_textures
  model = Sketchup.active_model
  sel = Sketchup.active_model.selection
  entities = sel.empty? ? model.active_entities.to_a : sel.to_a
  projective = find_projective_textures(entities)
  sel.clear
  sel.add(projective)
end # def select_projective_textures


def self.find_projective_textures(entities)
  projective = []
  faces = entities.find_all{|e| e.is_a?(Sketchup::Face) && (e.material && e.material.materialType>0 || e.back_material && e.back_material.materialType>0)}
  projective = faces.find_all{|f|
    texture_distorted?(f, true) ||
    texture_distorted?(f, false)
  }
  return projective
end # def find_projective_textures


# Test if a face has a projective (not affine) distortion
#   based on Tomasz Marek's su2kt.rb: SU2KT.texture_distorted
#   SketchUp's texturewriter writes a unique file for projective distortions.
#   Thus a projective texture gets a different handle, 
#   while two affine textures get the same handle.
# @param [Sketchup::Face] face
# @param [Boolean] front whether to check the front material or back material
#
def self.texture_distorted?(face, front=true)
  return false if face.class!=Sketchup::Face
	distorted=false
	temp_tw = Sketchup.create_texture_writer
  ents = Sketchup.active_model.active_entities
	Sketchup.active_model.start_operation "Group"
	group = ents.add_group
	group.material = (front==true)? face.material : face.back_material
	group_handle = temp_tw.load(group)
	temp_handle = temp_tw.load(face, front)
	ents.erase_entities(group)
	Sketchup.undo
	distorted = true if temp_handle != group_handle
	temp_tw = nil
	return distorted
end


end # module AE::FindProjectiveTextures


end # module AE



file_loaded(File.basename(__FILE__))
